import numpy as np
import gzip
import csv
import datetime

from NegativeCorrelationLearning import ModularNCLCNN

image_shape=(64,64)

num_classes=3036
total_test_data=133584
total_training_data=473616
batches=50
batch_size=64
num_experts=12

def get_data(index, test_data=False,jis_code=False):
    # Test data will return batch with random data order, where training data will move sequentially
    if test_data:
        #Randint is(inclusive,inclusive) whereas total_test_data is an exclusive range
        path='/cluster/home/kevinah/JapaneseCNN/OCR_test_data/%i.gz'%index

    else:
        path='/cluster/home/kevinah/JapaneseCNN/OCR_training_data/%i.gz'%index

    with gzip.open(path,'r') as reader:
        target=int(reader.readline()[1:-2])
        if jis_code:
            for last in reader:
                pass
            JIScode = last[4:-3]
    if jis_code:
        return {'image':np.loadtxt(path),'target':target,'JIScode':JIScode}
    else:
        return {'image':np.loadtxt(path),'target':target}

def get_batch(batch_size,test_data=False):
    batch=np.zeros((batch_size,1,image_shape[0],image_shape[1]))
    target=np.zeros(batch_size)
    if test_data:
        total=total_test_data
    else:
        total=total_training_data
    order=np.random.permutation(total)
    if test_data:
        length=total_test_data
    else:
        length=total_training_data

    for i in range(0,length,batch_size):# One batch per iter
        count=0
        for j in range(batch_size):# One image per iter
            if i+j>=total:
                break
            data=get_data(order[i+j],test_data=test_data)
            batch[count] = data['image']
            target[count]=data['target']
            count+=1
        yield (batch, target)


if __name__ == '__main__':
    model = ModularNCLCNN(image_shape, num_classes,num_experts).cuda(ModularNCLCNN.device)
    model.load()

    res =[] 
    for i in range(model.max_epoch):
        res.append(model.accuracy(get_batch(batch_size, test_data=True),forgiveness=i+1))
    
    with open('/cluster/home/kevinah/JapaneseCNN/Models/NCL_M1_uncertainty.csv','a',newline='') as file:
        writer=csv.writer(file)
        writer.writerow(res)
    
